/*
*   whoami.c 1.0 94/09/01
*   94/09/01 john h. dubois iii (john@armory.com)
*/

#include <pwd.h>
#include <grp.h>

unsigned short getuid ();
unsigned short geteuid ();
unsigned short getgid ();
unsigned short getegid ();
char *ltoa(long inval, char *string, int radix);

#define WriteString(S,fd) write(fd,S,strlen(S))
#define ErrExit(Message) WriteString(Message,2),exit(1)

char Help[] =
"whoami: print user or group IDs.\n"\
"Usage: whoami [huUgGns]\n"\
"h: Print this help.\n"\
"u: Print user ID.\n"\
"U: Print effective user ID.\n"\
"g: Print group ID.\n"\
"G: Print effective group ID.\n"\
"n: Switch to printing numeric IDs.\n"\
"s: Switch back to printing string IDs.\n"\
"If no arguments are given, the default of 'u' is used.\n"\
"To print a numeric user ID, use 'nu'.\n"\
"If the string form of an ID is not found, the numeric form is printed in\n"\
"its place.\n"\
"Example: to print each ID, with the numeric followed by the string form\n"\
"printed for each:\n"\
"whoami nusunUsUngsgnGsG\n";

char *Comment = "@(#) whoami 1.1 96/07/03";

main(int argc,char *argv[])
{
    int ArgInd;
    char *S;
    int Numeric = 0;
    unsigned short id;
    char NumID[11];	/* make it handle a 32-bit int just in case */
    struct passwd *Pass;
    struct group *Grp;
    char *StringName;

    if (argc == 1) {
	argv[0] = "u";
	ArgInd = 0;
    }
    else
	ArgInd = 1;
    for (; ArgInd < argc; ArgInd++)
	for (S = argv[ArgInd]; *S; S++) {
	    switch (*S) {
	    case 'u':
		id = getuid();
		break;
	    case 'U':
		id = geteuid();
		break;
	    case 'g':
		id = getgid();
		break;
	    case 'G':
		id = getegid();
		break;
	    case 'n':
		Numeric = 1;
		continue;
	    case 's':
		Numeric = 0;
		continue;
	    case '-':
		continue;
	    case 'h':
		WriteString(Help,1);
		exit(0);
	    default:
		ErrExit("Usage: whoami [hnsuUgG]\nUse h for help.\n");
		exit(1);
	    }
	    StringName = (char *) 0;
	    if (!Numeric) {
		if (*S == 'u' || *S == 'U') {
		    setpwent();
		    if (Pass = getpwuid((int) id))
			StringName = Pass->pw_name;
		}
		else {
		    setgrent();
		    if (Grp = getgrgid((int) id))
			StringName = Grp->gr_name;		
		}
	    }
	    WriteString(StringName ? StringName : ltoa((long) id,NumID,10),1);
	    write(1,"\n",1);
	}
    endpwent();
    endgrent();
    exit(0);
}
